<?php
/**
 * @package ats
 * @copyright Copyright (c)2011-2013 Nicholas K. Dionysopoulos / AkeebaBackup.com
 * @license GNU GPL v3 or later
 */

// No direct access
defined('_JEXEC') or die;

class AtsModelCpanels extends FOFModel
{
    /** @var string The root of the database installation files */
	private $dbFilesRoot = '/components/com_ats/sql/';

	/** @var array If any of these tables is missing we run the install SQL file and ignore the $dbChecks array */
	private $dbBaseCheck = array(
		'tables' => array(
			'ats_tickets', 'ats_posts', 'ats_attachments',
		),
		'file' => 'install/mysql/install.sql'
	);

	/** @var array Database update checks */
	private $dbChecks = array(
		array(
			'table' => 'ats_managernotes',
			'field' => '',
			'files' =>array(
				'updates/mysql/1.0.0-2012-07-12.sql',
			)
		),
		
		array(
			'table' => 'ats_cannedreplies',
			'field' => '',
			'files' =>array(
				'updates/mysql/1.0.0-2012-07-12.sql',
			)
		),
		
		array(
			'table' => 'ats_emailtemplates',
			'field' => '',
			'files' =>array(
				'updates/mysql/1.0.0-2012-10-31.sql',
			)
		),
		
		array(
			'table' => 'ats_managernotes',
			'field' => 'note_html',
			'files' =>array(
				'updates/mysql/1.0.0-2012-10-31.sql',
			)
		),
		
		array(
			'table' => 'ats_posts',
			'field' => 'origin',
			'files' =>array(
				'updates/mysql/1.0.0-2012-10-31.sql',
			)
		),
		
		array(
			'table' => 'ats_posts',
			'field' => 'email_uid',
			'files' =>array(
				'updates/mysql/1.0.0-2012-10-31.sql',
			)
		),
		
		array(
			'table' => 'ats_attachments',
			'field' => 'origin',
			'files' =>array(
				'updates/mysql/1.0.0-2012-10-31.sql',
			)
		),
		
		array(
			'table' => 'ats_credittransactions',
			'field' => '',
			'files' =>array(
				'updates/mysql/1.0.0-2012-11-12.sql',
			)
		),
		
		array(
			'table' => 'ats_creditconsumptions',
			'field' => '',
			'files' =>array(
				'updates/mysql/1.0.0-2012-11-12.sql',
			)
		),
		
		array(
			'table' => 'ats_offlineschedules',
			'field' => '',
			'files' =>array(
				'updates/mysql/1.0.0-2013-03-07.sql',
			)
		),
		
		array(
			'table' => 'ats_tickets',
			'field' => 'timespent',
			'files' =>array(
				'updates/mysql/1.0.0-2013-04-03.sql',
			)
		),
		
		array(
			'table' => 'ats_posts',
			'field' => 'timespent',
			'files' =>array(
				'updates/mysql/1.0.0-2013-04-03.sql',
			)
		),
		
        array(
			'table' => 'ats_tickets',
			'field' => 'assigned_to',
			'files' =>array(
				'updates/mysql/1.1.0-2013-04-08.sql',
			)
		),
		
        array(
			'table' => 'ats_tickets',
			'field' => 'params',
			'files' =>array(
				'updates/mysql/1.1.0-2013-04-25.sql',
			)
		),
		
        array(
			'table' => 'ats_customfields',
			'field' => '',
			'files' =>array(
				'updates/mysql/1.1.0-2013-04-25.sql',
			)
		),
		
        array(
			'table' => 'ats_customfields_cats',
			'field' => '',
			'files' =>array(
				'updates/mysql/1.1.0-2013-04-25.sql',
			)
		),
		
        array(
			'table' => 'ats_tickets',
			'field' => 'ats_bucket_id',
			'files' =>array(
				'updates/mysql/1.1.0-2013-05-03.sql',
			)
		),
		
        array(
			'table' => 'ats_buckets',
			'field' => '',
			'files' =>array(
				'updates/mysql/1.1.0-2013-05-03.sql',
			)
		),
		
        array(
			'table' => 'ats_tickets',
			'field' => 'priority',
			'files' =>array(
				'updates/mysql/1.1.0-2013-05-08.sql',
			)
		),
		
        array(
			'table' => 'ats_tickets',
			'field' => 'rating',
			'files' =>array(
				'updates/mysql/1.2.0-2013-06-05.sql',
			)
		),
	);

	/**
	 * Checks the database for missing / outdated tables using the $dbChecks
	 * data and runs the appropriate SQL scripts if necessary.
	 *
	 * @return AkeebasubsModelCpanels
	 */
	public function checkAndFixDatabase()
	{
		$db = $this->getDbo();

		// Initialise
		$tableFields = array();
		$sqlFiles = array();

		// Get a listing of database tables known to Joomla!
		$allTables = $db->getTableList();
		$dbprefix = JFactory::getConfig()->get('dbprefix', '');

		// Perform the base check. If any of these tables is missing we have to run the installation SQL file
		if(!empty($this->dbBaseCheck)) {
			foreach($this->dbBaseCheck['tables'] as $table)
			{
				$tableName = $dbprefix . $table;
				$check = in_array($tableName, $allTables);
				if (!$check) break;
			}

			if (!$check)
			{
				$sqlFiles[] = JPATH_ADMINISTRATOR . $this->dbFilesRoot . $this->dbBaseCheck['file'];
			}
		}

		// If the base check was successful and we have further database checks run them
		if (empty($sqlFiles) && !empty($this->dbChecks)) foreach($this->dbChecks as $dbCheck)
		{
			// Always check that the table exists
			$tableName = $dbprefix . $dbCheck['table'];
			$check = in_array($tableName, $allTables);

			// If the table exists and we have a field, check that the field exists too
			if (!empty($dbCheck['field']) && $check)
			{
				if (!array_key_exists($tableName, $tableFields))
				{
					$tableFields[$tableName] = $db->getTableColumns('#__' . $dbCheck['table'], true);
				}

				if (is_array($tableFields[$tableName]))
				{
					$check = array_key_exists($dbCheck['field'], $tableFields[$tableName]);
				}
				else
				{
					$check = false;
				}
			}

			// Something's missing. Add the file to the list of SQL files to run
			if (!$check)
			{
				foreach ($dbCheck['files'] as $file)
				{
					$sqlFiles[] = JPATH_ADMINISTRATOR . $this->dbFilesRoot . $file;
				}
			}
		}

		// If we have SQL files to run, well, RUN THEM!
		if (!empty($sqlFiles))
		{
			JLoader::import('joomla.filesystem.file');
			foreach($sqlFiles as $file)
			{
				$sql = JFile::read($file);
				if($sql) {
					$commands = explode(';', $sql);
					foreach($commands as $query) {
						$db->setQuery($query);
						$db->execute();
					}
				}
			}
            
            // Finally clear FOF's cache
            $fof_platform = FOFPlatform::getInstance();
            if (method_exists($fof_platform, 'clearCache'))
            {
                FOFPlatform::getInstance()->clearCache();
            }
		}

		return $this;
	}
    
	/**
	 * Does the user needs to enter a Download ID?
	 *
	 * @return boolean
	 */
	public function needsDownloadID()
	{
		JLoader::import('joomla.application.component.helper');

		// Do I need a Download ID?
		$ret = false;
		$params = JComponentHelper::getParams('com_ats');
		if(version_compare(JVERSION, '3.0', 'ge')) {
			$dlid = $params->get('downloadid', '');
		} else {
			$dlid = $params->getValue('downloadid', '');
		}
		if(!preg_match('/^([0-9]{1,}:)?[0-9a-f]{32}$/i', $dlid)) {
			$ret = true;
		}

		return $ret;
	}
}